<?php

namespace Drupal\Tests\homebox\Functional;

/**
 * Functional Homebox tests, to verify the modules functionality.
 *
 * @group homebox
 */
class AccessTests extends FunctionalHomeboxTestBase {

  /**
   * Tests all homebox routes.
   */
  public function testHomeboxRoutes() {
    $this->homeboxTestHelper->createHomebox('test', 'test', TRUE, TRUE, [], 'layout_onecol', [], '/custom-path');

    // Check homebox menu route:
    $this->drupalGet('/homebox');
    $this->assertSession()->statusCodeEquals(200);

    // Check default dashboard_page route:
    $this->drupalGet('/homebox/test');
    $this->assertSession()->statusCodeEquals(200);

    // Check dashboard_page with uid:
    $this->drupalGet('/homebox/test/1');
    $this->assertSession()->statusCodeEquals(200);

    // Check dashboard_user_tab route:
    $this->drupalGet('/user/1/homebox/test');
    $this->assertSession()->statusCodeEquals(200);

    // Check dashboard_reset route:
    $this->drupalGet('/homebox/test/1/reset');
    $this->assertSession()->statusCodeEquals(200);

    // Check admin_library route:
    $this->drupalGet('/homebox/block/library/test');
    $this->assertSession()->statusCodeEquals(200);

    // Check homebox add route:
    $this->drupalGet('/homebox/add');
    $this->assertSession()->statusCodeEquals(200);

    // Check homebox edit route:
    $this->drupalGet('/homebox/test/edit');
    $this->assertSession()->statusCodeEquals(200);

    // Check homebox delete route:
    $this->drupalGet('/homebox/test/delete');
    $this->assertSession()->statusCodeEquals(200);

    // Check homebox dashboard_reset route:
    $this->drupalGet('/homebox/test/1/reset');
    $this->assertSession()->statusCodeEquals(200);

    // Check homebox_layout.add_form:
    $this->drupalGet('/admin/content/homebox/homebox_layout/add/test');
    $this->assertSession()->statusCodeEquals(200);

    // Check homebox_layout.add_page:
    $this->drupalGet('/admin/content/homebox/homebox_layout/add');
    $this->assertSession()->statusCodeEquals(200);

    // Check custom page path:
    $this->drupalGet('/custom-path');
    $this->assertSession()->statusCodeEquals(200);
  }

  /**
   * Tests Dashboard Page and Dashboard User Tab Page Enabled / Disabled access.
   */
  public function testDashboardPagesAccess() {
    // Test Homebox with enabled dashboard page, but disabled user tab page:
    $this->homeboxTestHelper->createHomebox('test', 'test', TRUE, FALSE, [], 'layout_onecol', [], '/custom-path');
    // Go to the dashboard page via all routes:
    $this->drupalGet('/homebox/test/1');
    $this->assertSession()->statusCodeEquals(200);
    $this->drupalGet('/homebox/test');
    $this->assertSession()->statusCodeEquals(200);
    $this->drupalGet('/custom-path');
    $this->assertSession()->statusCodeEquals(200);
    // Since the user tab page is disabled, we should not have access:
    $this->drupalGet('/user/1/homebox/test');
    $this->assertSession()->statusCodeEquals(403);

    // Test Homebox with disabled dashboard page, but enabled user tab page:
    $this->homeboxTestHelper->createHomebox('test2', 'test2', FALSE, TRUE, [], 'layout_onecol', [], '/custom-path-second');
    // Go to the user tab page:
    $this->drupalGet('/user/1/homebox/test2');
    $this->assertSession()->statusCodeEquals(200);
    // Since the dashboard page is disabled.
    // We should not have access via any route:
    $this->drupalGet('/homebox/test2/1');
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet('/homebox/test2');
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet('/custom-path-second');
    $this->assertSession()->statusCodeEquals(404);
  }

  /**
   * Tests the 'use homebox' permission.
   */
  public function testUseHomeboxPermission() {
    $this->homeboxTestHelper->createHomebox('test', 'test', TRUE, TRUE, [], 'layout_onecol', [], '/custom-path');
    $this->drupalLogout();
    $mockUser = $this->drupalCreateUser(['use homebox']);
    $mockUid = $mockUser->id();
    $user = $this->drupalCreateUser(['use homebox']);
    $uid = $user->id();
    $this->drupalLogin($user);
    // Check access:
    $this->drupalGet('/homebox/test');
    $this->assertSession()->statusCodeEquals(200);
    $this->drupalGet('/homebox/test/' . $uid);
    $this->assertSession()->statusCodeEquals(200);
    $this->drupalGet('/custom-path');
    $this->assertSession()->statusCodeEquals(200);
    $this->drupalGet('/user/' . $uid . '/homebox/test');
    $this->assertSession()->statusCodeEquals(200);
    // The user should not add, edit or delete homeboxes:
    $this->drupalGet('/homebox/add');
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet('/homebox/test/edit');
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet('/homebox/test/delete');
    $this->assertSession()->statusCodeEquals(403);
    // The user should not have access to admin settings:
    $this->drupalGet('/homebox/test/settings');
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet('/homebox/block/library/test');
    $this->assertSession()->statusCodeEquals(403);
    // The user should not have access to other peoples homeboxes:
    $this->drupalGet('/homebox/test/' . $mockUid);
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet('/user/' . $mockUid . '/homebox/test');
    $this->assertSession()->statusCodeEquals(403);
    // The user can reset their own homebox but not others:
    $this->drupalGet('/homebox/test/' . $uid . '/reset');
    $this->assertSession()->statusCodeEquals(200);
    $this->drupalGet('/homebox/test/' . $mockUid . '/reset');
    $this->assertSession()->statusCodeEquals(403);
    // If the pages are disabled, we should not have access to them, under any
    // circumstances:
    $this->homeboxTestHelper->createHomebox('test2', 'test2', FALSE, FALSE, [], 'layout_onecol', [], '/custom-path2');
    $this->drupalGet('/homebox/test2');
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet('/homebox/test2/' . $uid);
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet('/custom-path2');
    $this->assertSession()->statusCodeEquals(404);
    $this->drupalGet('/user/' . $uid . '/homebox/test2');
    $this->assertSession()->statusCodeEquals(403);
  }

  /**
   * Tests the 'bypass homeboxes access' permission.
   */
  public function testBypassHomeboxesAccessPermission() {
    $this->homeboxTestHelper->createHomebox('test', 'test', TRUE, TRUE, [], 'layout_onecol', [], '/custom-path');
    $this->drupalLogout();
    $mockUser = $this->drupalCreateUser(['use homebox']);
    $mockUid = $mockUser->id();
    $user = $this->drupalCreateUser(['bypass homeboxes access']);
    $uid = $user->id();
    $this->drupalLogin($user);
    // The user should have access to all homebox relevant routes:
    $this->drupalGet('/homebox/test');
    $this->assertSession()->statusCodeEquals(200);
    $this->drupalGet('/homebox/test/' . $uid);
    $this->assertSession()->statusCodeEquals(200);
    $this->drupalGet('/custom-path');
    $this->assertSession()->statusCodeEquals(200);
    $this->drupalGet('/user/' . $uid . '/homebox/test');
    $this->assertSession()->statusCodeEquals(200);
    // The user should have access to other peoples homeboxes:
    $this->drupalGet('/homebox/test/' . $mockUid);
    $this->assertSession()->statusCodeEquals(200);
    $this->drupalGet('/user/' . $mockUid . '/homebox/test');
    $this->assertSession()->statusCodeEquals(200);
    // The user should not have access to admin settings:
    $this->drupalGet('/homebox/test/settings');
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet('/homebox/block/library/test');
    $this->assertSession()->statusCodeEquals(403);
    // The user should not add, edit or delete homeboxes:
    $this->drupalGet('/homebox/add');
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet('/homebox/test/edit');
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet('/homebox/test/delete');
    $this->assertSession()->statusCodeEquals(403);
    // The user can reset their own homebox and others:
    $this->drupalGet('/homebox/test/' . $uid . '/reset');
    $this->assertSession()->statusCodeEquals(200);
    $this->drupalGet('/homebox/test/' . $mockUid . '/reset');
    $this->assertSession()->statusCodeEquals(200);
    // If the pages are disabled, we should not have access to them, under any
    // circumstances:
    $this->homeboxTestHelper->createHomebox('test2', 'test2', FALSE, FALSE, [], 'layout_onecol', [], '/custom-path2');
    $this->drupalGet('/homebox/test2');
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet('/homebox/test2/' . $uid);
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet('/custom-path2');
    $this->assertSession()->statusCodeEquals(404);
    $this->drupalGet('/user/' . $uid . '/homebox/test2');
    $this->assertSession()->statusCodeEquals(403);
  }

  /**
   * Tests the 'administer homebox types' permission.
   */
  public function testAdministerHomeboxPermission() {
    $this->homeboxTestHelper->createHomebox('test', 'test', TRUE, TRUE, [], 'layout_onecol', [], '/custom-path');
    $this->drupalLogout();
    $mockUser = $this->drupalCreateUser(['use homebox']);
    $mockUid = $mockUser->id();
    $user = $this->drupalCreateUser(['administer homebox types']);
    $uid = $user->id();
    $this->drupalLogin($user);
    // The user should only have access to the homebox settings:
    $this->drupalGet('/homebox/test');
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet('/homebox/test/' . $uid);
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet('/custom-path');
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet('/user/' . $uid . '/homebox/test');
    $this->assertSession()->statusCodeEquals(403);
    // The user should not have access to other peoples homeboxes:
    $this->drupalGet('/homebox/test/' . $mockUid);
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet('/user/' . $mockUid . '/homebox/test');
    $this->assertSession()->statusCodeEquals(403);
    // The user should not add, edit or delete homeboxes:
    $this->drupalGet('/homebox/add');
    $this->assertSession()->statusCodeEquals(200);
    $this->drupalGet('/homebox/test/edit');
    $this->assertSession()->statusCodeEquals(200);
    $this->drupalGet('/homebox/test/delete');
    $this->assertSession()->statusCodeEquals(200);
    // The user should have access to admin settings:
    $this->drupalGet('/homebox/test/settings');
    $this->assertSession()->statusCodeEquals(200);
    $this->drupalGet('/homebox/block/library/test');
    $this->assertSession()->statusCodeEquals(200);
    // The user can not reset their own homebox nor others:
    $this->drupalGet('/homebox/test/' . $uid . '/reset');
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet('/homebox/test/' . $mockUid . '/reset');
    $this->assertSession()->statusCodeEquals(403);
    // If the pages are disabled, we should not have access to them, under any
    // circumstances:
    $this->homeboxTestHelper->createHomebox('test2', 'test2', FALSE, FALSE, [], 'layout_onecol', [], '/custom-path2');
    $this->drupalGet('/homebox/test2');
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet('/homebox/test2/' . $uid);
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet('/custom-path2');
    $this->assertSession()->statusCodeEquals(404);
    $this->drupalGet('/user/' . $uid . '/homebox/test2');
    $this->assertSession()->statusCodeEquals(403);
  }

  /**
   * Tests homebox access logic as an anonymous user.
   */
  public function testAccessAsAnonymous() {
    $this->homeboxTestHelper->createHomebox('test', 'test', TRUE, TRUE, [], 'layout_onecol', [], '/custom-path');
    $this->drupalLogout();
    $mockUser = $this->drupalCreateUser(['use homebox']);
    $mockUid = $mockUser->id();

    // There should be no Homebox for an anonymous user:
    $this->drupalGet('/homebox/test');
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet('/homebox/test/0');
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet('/custom-path');
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet('/user/0/homebox/test');
    $this->assertSession()->statusCodeEquals(403);
    // The user should not have access to other peoples homeboxes:
    $this->drupalGet('/homebox/test/' . $mockUid);
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet('/user/' . $mockUid . '/homebox/test');
    $this->assertSession()->statusCodeEquals(403);
    // The user should not add, edit or delete homeboxes:
    $this->drupalGet('/homebox/add');
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet('/homebox/test/edit');
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet('/homebox/test/delete');
    $this->assertSession()->statusCodeEquals(403);
    // The user should not have access to admin settings:
    $this->drupalGet('/homebox/test/settings');
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet('/homebox/block/library/test');
    $this->assertSession()->statusCodeEquals(403);
    // There should be no reset page for an anonymous user:
    $this->drupalGet('/homebox/test/0/reset');
    $this->assertSession()->statusCodeEquals(403);
    // The anonymous user shouldn't be able to reset other peoples homebox:
    $this->drupalGet('/homebox/test/' . $mockUid . '/reset');
    $this->assertSession()->statusCodeEquals(403);
    // If the pages are disabled, we should not have access to them, under any
    // circumstances:
    $this->homeboxTestHelper->createHomebox('test2', 'test2', FALSE, FALSE, [], 'layout_onecol', [], '/custom-path2');
    $this->drupalGet('/homebox/test2');
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet('/custom-path2');
    $this->assertSession()->statusCodeEquals(404);
  }

  /**
   * Tests homebox disallowed access, set internally in the homebox.
   */
  public function testInternalHomeboxAllowAccess() {
    $this->createRole(['use homebox'], 'test_role', 'test_role');
    $this->homeboxTestHelper->createHomebox('test', 'test', TRUE, TRUE, ['test_role'], 'layout_onecol', [], '/custom-path');
    $this->drupalLogout();
    $mockUser = $this->drupalCreateUser(['use homebox']);
    $mockUser->addRole('test_role');
    $mockUser->save();
    $mockUid = $mockUser->id();

    // Login as user without the right role:
    $user = $this->drupalCreateUser(['use homebox']);
    $uid = $user->id();
    $this->drupalLogin($user);

    // The user should not have access to the Dashboard Form:
    $this->drupalGet('/homebox/test');
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet('/homebox/test/' . $uid);
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet('/custom-path');
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet('/user/' . $uid . '/homebox/test');
    $this->assertSession()->statusCodeEquals(403);
    // The user should not have access to other peoples homeboxes:
    $this->drupalGet('/homebox/test/' . $mockUid);
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet('/user/' . $mockUid . '/homebox/test');
    $this->assertSession()->statusCodeEquals(403);
    // The user should not add, edit or delete homeboxes:
    $this->drupalGet('/homebox/add');
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet('/homebox/test/edit');
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet('/homebox/test/delete');
    $this->assertSession()->statusCodeEquals(403);
    // The user should not have access to admin settings:
    $this->drupalGet('/homebox/test/settings');
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet('/homebox/block/library/test');
    $this->assertSession()->statusCodeEquals(403);
    // The user can not reset their own homebox nor others:
    $this->drupalGet('/homebox/test/' . $uid . '/reset');
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet('/homebox/test/' . $mockUid . '/reset');
    $this->assertSession()->statusCodeEquals(403);
    // If the pages are disabled, we should not have access to them, under any
    // circumstances:
    $this->homeboxTestHelper->createHomebox('test2', 'test2', FALSE, FALSE, [], 'layout_onecol', [], '/custom-path2');
    $this->drupalGet('/homebox/test2');
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet('/homebox/test2/' . $uid);
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet('/custom-path2');
    $this->assertSession()->statusCodeEquals(404);
    $this->drupalGet('/user/' . $uid . '/homebox/test2');
    $this->assertSession()->statusCodeEquals(403);

    // Login as a user, who has the right permissions:
    $this->drupalLogout();
    $this->drupalLogin($mockUser);

    // The user should have access to the Dashboard Form:
    $this->drupalGet('/homebox/test');
    $this->assertSession()->statusCodeEquals(200);
    $this->drupalGet('/homebox/test/' . $mockUid);
    $this->assertSession()->statusCodeEquals(200);
    $this->drupalGet('/custom-path');
    $this->assertSession()->statusCodeEquals(200);
    $this->drupalGet('/user/' . $mockUid . '/homebox/test');
    $this->assertSession()->statusCodeEquals(200);
    // The user should not have access to other peoples homeboxes:
    $this->drupalGet('/homebox/test/' . $uid);
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet('/user/' . $uid . '/homebox/test');
    $this->assertSession()->statusCodeEquals(403);
    // The user should not add, edit or delete homeboxes:
    $this->drupalGet('/homebox/add');
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet('/homebox/test/edit');
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet('/homebox/test/delete');
    $this->assertSession()->statusCodeEquals(403);
    // The user should not have access to admin settings:
    $this->drupalGet('/homebox/test/settings');
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet('/homebox/block/library/test');
    $this->assertSession()->statusCodeEquals(403);
    // The user can reset their own homebox but not others:
    $this->drupalGet('/homebox/test/' . $mockUid . '/reset');
    $this->assertSession()->statusCodeEquals(200);
    $this->drupalGet('/homebox/test/' . $uid . '/reset');
    $this->assertSession()->statusCodeEquals(403);
    // If the pages are disabled, we should not have access to them, under any
    // circumstances:
    $this->drupalGet('/homebox/test2');
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet('/homebox/test2/' . $uid);
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet('/custom-path2');
    $this->assertSession()->statusCodeEquals(404);
    $this->drupalGet('/user/' . $uid . '/homebox/test2');
    $this->assertSession()->statusCodeEquals(403);
  }

}
