<?php

namespace Drupal\Tests\homebox\Functional;

use Drupal\Tests\BrowserTestBase;

/**
 * This class provides methods specifically for testing something.
 *
 * @group homebox
 */
class HomeboxSettingsTest extends BrowserTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'homebox',
    'options',
  ];

  /**
   * A user with authenticated permissions.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $user;

  /**
   * A user with admin permissions.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $adminUser;

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->config('system.site')->set('page.front', '/test-page')->save();
    $this->user = $this->drupalCreateUser([]);
    $this->adminUser = $this->drupalCreateUser([]);
    $this->adminUser->addRole($this->createAdminRole('admin', 'admin'));
    $this->adminUser->save();
    $this->drupalLogin($this->adminUser);
  }

  /**
   * Tests the functionality of the settings page.
   */
  public function testSettingsChange() {
    $session = $this->assertSession();
    $page = $this->getSession()->getPage();
    // Go to the settings page and change the settings.
    $this->drupalGet('/admin/structure/homebox_type/settings');
    $session->statusCodeEquals(200);
    $session->fieldValueEquals('portletColors', 'grey|Grey\ngreen|Green\nyellow|Yellow\nred|Red\nblue|Blue');
    $page->fillField('portletColors', '#444444|Dark Grey\n#00aa00|Green');
    // Save the settings and check if they are still there.
    $page->pressButton('edit-submit');
    $session->statusCodeEquals(200);
    $session->fieldValueEquals('portletColors', '#444444|Dark Grey\n#00aa00|Green');
  }

}
