<?php

namespace Drupal\Tests\homebox\Functional;

use Drupal\homebox\Entity\Homebox;

/**
 * A Test Helper class for Homebox Tests.
 */
class HomeboxTestHelper {

  /**
   * Creates a homebox entity.
   *
   * @param string $id
   *   The homebox id.
   * @param string $label
   *   The homebox label.
   * @param bool $pageEnabled
   *   The homebox pageEnabled variable.
   * @param bool $userTabEnabled
   *   The homebox userTabEnabled variable.
   * @param array $restrictedRoles
   *   The unallowed homebox access roles.
   * @param string $layoutId
   *   The homebox layout id.
   * @param array $blocks
   *   The homebox blocks.
   * @param string $pagePath
   *   The homebox custom page path.
   */
  public function createHomebox(string $id, string $label, bool $pageEnabled = FALSE, bool $userTabEnabled = FALSE, array $restrictedRoles = [], string $layoutId = '', array $blocks = [], string $pagePath = '') {
    $homeboxType = Homebox::create(
          [
            'id' => $id,
            'label' => $label,
            'restrictedRoles' => $restrictedRoles,
            'layoutId' => $layoutId,
            'blocks' => $blocks,
            'pageEnabled' => $pageEnabled,
            'userTabEnabled' => $userTabEnabled,
            'pagePath' => $pagePath,
          ]
      )->save();
    \Drupal::service('router.builder')->rebuild();
    return $homeboxType;
  }

  /**
   * This function will create a pseudo block array.
   *
   * These pseudo blocks are saved for each homebox entity, see
   * https://www.drupal.org/project/homebox/issues/3266328.
   *
   * @param string $id
   *   The pseudo block id.
   * @param string $title
   *   The pseudo block title.
   * @param int $weight
   *   The pseudo block weight.
   * @param string $region
   *   The pseudo block region.
   * @param bool $status
   *   The pseudo block status.
   */
  public function createPseudoBlock(string $id, string $title, int $weight, string $region, bool $status) {
    return [
      'id' => $id,
      'title' => $title,
      'weight' => $weight,
      'region' => $region,
      'status' => $status,
    ];
  }

}
